/*
 * misc-lzma.c 
 * LZMA decompression module 
 *
 * Copyright (C) 2004 Streetdata Pty Ltd.
 * Copyright (C) 2005 Marek Michalkiewicz
 */

//#define puts prom_printf

#define memzero(d, count) (memset((d), 0, (count)))

#define __ptr_t void *
#define NULL ((void *) 0)

typedef unsigned char  uch;
typedef unsigned short ush;
typedef unsigned long  ulg;

extern char input_data[];
extern char input_data_end[];

static uch *output_data;
//static ulg output_ptr;
//static ulg bytes_out;

#define _LZMA_PROB32
#include "LzmaDecode.c"
#include <stdio.h>

static void error(char *x)
{
	//int ptr;

	//puts("\n\n");
	//puts(x);
	//puts("\n\n -- System halted");

	while(1);	/* Halt */
}


#define MAX_LC_PLUS_LP 3
#define LZMA_BUF_SIZE (LZMA_BASE_SIZE + ((LZMA_LIT_SIZE) << (MAX_LC_PLUS_LP)))

#define         BASE                    0xbc000003

/* distance in bytes between two serial registers */
#define         REG_OFFSET              4


/* register offset */
#define         OFS_RCV_BUFFER          (0*REG_OFFSET)
#define         OFS_TRANS_HOLD          (0*REG_OFFSET)
#define         OFS_SEND_BUFFER         (0*REG_OFFSET)
#define         OFS_INTR_ENABLE         (1*REG_OFFSET)
#define         OFS_INTR_ID             (2*REG_OFFSET)
#define         OFS_DATA_FORMAT         (3*REG_OFFSET)
#define         OFS_LINE_CONTROL        (3*REG_OFFSET)
#define         OFS_MODEM_CONTROL       (4*REG_OFFSET)
#define         OFS_RS232_OUTPUT        (4*REG_OFFSET)
#define         OFS_LINE_STATUS         (5*REG_OFFSET)
#define         OFS_MODEM_STATUS        (6*REG_OFFSET)
#define         OFS_RS232_INPUT         (6*REG_OFFSET)
#define         OFS_SCRATCH_PAD         (7*REG_OFFSET)

#define         OFS_DIVISOR_LSB         (0*REG_OFFSET)
#define         OFS_DIVISOR_MSB         (1*REG_OFFSET)

#define         UART16550_BAUD_2400             2400
#define         UART16550_BAUD_4800             4800
#define         UART16550_BAUD_9600             9600
#define         UART16550_BAUD_19200            19200
#define         UART16550_BAUD_38400            38400
#define         UART16550_BAUD_57600            57600
#define         UART16550_BAUD_115200           115200

#define         UART16550_PARITY_NONE           0
#define         UART16550_PARITY_ODD            0x08
#define         UART16550_PARITY_EVEN           0x18
#define         UART16550_PARITY_MARK           0x28
#define         UART16550_PARITY_SPACE          0x38

#define         UART16550_DATA_5BIT             0x0
#define         UART16550_DATA_6BIT             0x1
#define         UART16550_DATA_7BIT             0x2
#define         UART16550_DATA_8BIT             0x3

#define         UART16550_STOP_1BIT             0x0
#define         UART16550_STOP_2BIT             0x4



/* memory-mapped read/write of the port */
#define         UART16550_READ(y)    (*((volatile unsigned char *)(BASE + y)))
#define         UART16550_WRITE(y, z)  ((*((volatile unsigned char *)(BASE + y))) = z)


static CProb lzma_buf[LZMA_BUF_SIZE];

void uart_init (void)
{
    // sysRegWrite(AR531X_RESET,
    // sysRegRead(AR531X_RESET) & ~(AR531X_RESET_UART0));

    // disable interrupts 
        UART16550_WRITE(OFS_LINE_CONTROL, 0x0);
	UART16550_WRITE(OFS_INTR_ENABLE, 0);

    // set up buad rate 
	unsigned int divisor;		
    //    u32 uart_clock_rate = ar531x_cpu_frequency() / 4;
	unsigned int base_baud = 0x2aea54;
        // set DIAB bit 
		UART16550_WRITE(OFS_LINE_CONTROL, 0x80);
        // set divisor 
		divisor = base_baud / UART16550_BAUD_115200;
	    UART16550_WRITE(OFS_DIVISOR_LSB, divisor & 0xff);
		UART16550_WRITE(OFS_DIVISOR_MSB, (divisor & 0xff00)>>8);
        // clear DIAB bit 
		UART16550_WRITE(OFS_LINE_CONTROL, 0x0);

    // set data format 
	UART16550_WRITE(OFS_DATA_FORMAT, UART16550_DATA_8BIT | UART16550_PARITY_NONE | UART16550_STOP_1BIT);
}


void print_uart (unsigned char * retezec)
{
unsigned char znak;
int i=0;
znak=retezec[i++];
    while (znak != 0x0)
    {	

	while ((UART16550_READ(OFS_LINE_STATUS) &0x20) == 0);	
	UART16550_WRITE(OFS_SEND_BUFFER,znak);
	znak=retezec[i++];
    }
}

void
decompress_kernel(void)
{
	uart_init();



//	print_uart("AHOJ\n");
//	UART16550_WRITE(OFS_SEND_BUFFER,'Y');

	//int lc, lp, pb, result;
	int result;
	unsigned int input_size, output_size;
	unsigned char *p;
	CLzmaDecoderState state;
	
	output_data = (char *)LOADADDR;
	//puts("Uncompressing [LZMA] ... ");

	p = (unsigned char *) input_data;

	if (LzmaDecodeProperties(&state.Properties, p, LZMA_PROPERTIES_SIZE)
	    != LZMA_RESULT_OK)
		error("bad properties");

	if (LzmaGetNumProbs(&state.Properties) > LZMA_BUF_SIZE)
		error("not enough memory");

	state.Probs = lzma_buf;

	input_size = input_data_end - input_data;

	output_size = p[8];
	output_size = (output_size << 8) + p[7];
	output_size = (output_size << 8) + p[6];
	output_size = (output_size << 8) + p[5];

	//if (LOADADDR < UNCOMPRESS_OUT + output_size)
	//	error("image too big");

#define LZMA_HEADER_SIZE 13
	p += LZMA_HEADER_SIZE;
	input_size -= LZMA_HEADER_SIZE;

	result = LzmaDecode(&state, p, input_size, &input_size,
			    output_data, output_size, &output_size);

	if (result != LZMA_RESULT_OK)
		error("decompression error");

	//puts(" done.\n");
	//return output_ptr;
}

